#!/usr/bin/env python
# -*- coding:Utf-8 -*-

## wallpabox.py - GUI - Choose and modify the wallpaper on a fluxbox or openbox desktop
#
# Copyright (C) 2008 Nicolas Doby <nicolas.doby@gmail.com>
#
# Licensed under the GNU General Public License Version 2
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import with_statement # for files
import gtk, pygtk, urllib
import os, sys, shutil
import gettext

pygtk.require('2.0')


# Set locales
gettext.install("wallpabox", "/usr/share/locale/")

# Return True if path is an image file
def is_image(path):
	return (path.lower().endswith('jpg') or path.lower().endswith('jpeg') or path.lower().endswith('png') or path.lower().endswith('bmp'))

class ChangeWallpaper(gtk.Window):

	command_arg = [" -f ", " -c ", " -t ", " -a ", " -r "]

	# Read the configfile and restore datas
	def read_configfile(self):
		with open(self.config_file, "r") as f:
			line = f.readline()
			while (not line == ""):
				line = line.strip()
				if(line.startswith("random=")):
					if(line[7:len(line)] == '1'):
						self.randomcheck.set_active(True)
				if(line.startswith("mode=")):
					self.combo.set_active(int(line[5:len(line)]))
				line = f.readline()

	# Save datas in order to restore for the next launch
	def save_config(self):
		if(self.must_save):
			print _("Save settings.")
			f = open(self.config_file, "w")
			random = "random="
			if(self.randomcheck.get_active()):
				random += "1"
			else:
				random += "0"
			mode = "mode="+str(self.combo.get_active())
			f.write("---Config file of Wallpabox---\n\n")
			f.write(random+"\n")
			f.write(mode+"\n")
			f.close()

	# Set the new wallpaper
	def set_wallpaper(self, index):
		# Get the combobox choice selection and add the argumment
		arg = self.command_arg[self.combo.get_active()]
		# Set random mode if the checkbutton is enable
		if(self.randomcheck.get_active()):
			arg += self.command_arg[len(self.command_arg)-1]
			os.popen("/usr/bin/fbsetbg"+ arg + self.directory)
			print _("Random wallpapers")
		else:
			# Test if a picture is selected
			if(index != -1):
				filename = "\"" + self.images_paths[index] + "\""
				if(os.path.exists(self.images_paths[index])):
					os.popen("/usr/bin/fbsetbg"+ arg + filename)
					print _("New wallpaper : ") + filename
				else:
					print _("The file \"%s\" does not exist.") % filename
					del self.images_paths[index]
					self.model.remove(self.model.get_iter(index))
		self.must_save = True

	# Callbacks #

	# Display a 'FileChooserDiablog'. The user can import one or more pictures.
	def select_file(self, widget):
		result = self.filesel.run()
		self.filesel.hide()
		if(result == gtk.RESPONSE_OK):
			filenames = self.filesel.get_filenames()
			self.add_files_to_iconview(filenames)
			

	# Add files in the self.iconview
	def add_files_to_iconview(self, filenames):
		for i in filenames:			
			# Check if a filename already exists
			temp = i.split('/')
			new_path = self.directory+temp[len(temp)-1]
			if new_path in self.images_paths:
				print _("The file \"%s\" has already been added.") % i
			else:
				try:
					# Copy the picture in the config folder (except the first launch)
					if(i != new_path):
						shutil.copy(i, new_path)
					image = gtk.gdk.pixbuf_new_from_file_at_size(new_path, 96, 96)
					print _("Add : ") + i
					self.images_paths.append(new_path)					
					self.model.append([image])
				except:
					print _("Error : %s is not a correct file.") % i

						
	# Set the selected image as the new wallpaper
	def on_apply_click(self, widget):
		widget.set_sensitive(False)
		liste = self.iconview.get_selected_items()
		if liste != []:
			index = self.iconview.get_selected_items()[0][0]
			self.set_wallpaper(index)
		else:
			self.set_wallpaper(-1)

	# Remove all selected images
	def on_remove_click(self, widget):
		widget.set_sensitive(False)
		liste = self.iconview.get_selected_items()
		model = self.iconview.get_model()
		if len(liste) != 0:
			for image in liste:				
				index = image[0]
				filename = self.images_paths[index]					
				model.remove(model.get_iter(index))
				os.remove(filename)
				del self.images_paths[index]
				print _("Remove : ") + filename


	def evnmt_delete(self, widget, evenement, donnees=None):
		self.save_config()
		return False

	def destroy(self, widget, donnees=None):
		self.save_config()
		gtk.main_quit()

	# Enable the apply and remove button if the user clicked on a thumbnail
	# Disable them if not
	def on_selection_changed(self, widget):
		liste = self.iconview.get_selected_items()
		if len(liste) != 0:
			if len(liste) == 1:			
				self.button_apply.set_sensitive(True)
			else:
				self.button_apply.set_sensitive(False)
			self.button_remove.set_sensitive(True)
		else:
			if(self.randomcheck.get_active() == False):
				self.button_apply.set_sensitive(False)
			self.button_remove.set_sensitive(False)

	# Set the new wallpaper if the user double click on a thumbnail
	def on_double_click(self, iconview, path):
		self.button_apply.emit("clicked")
		
	# Add wallpapers when drop in the self.iconview
	def receiveCallback(self, widget, context, x, y, selection, targetType, time):
		# Remove "file://" from the begining and "\r\t" from the end of the string
		data = urllib.url2pathname(selection.data)
		data = data.replace("\r\n", " ")
		data = data[7:len(selection.data)-2]
		# Split the string to have a list of paths
		paths = data.split(' file://')
		final_paths = []
		
		# Check if all the files are picture's types
		for p in paths :
			# Remove all spaces
			p = p.strip()
			if (is_image(p)):
				final_paths.append(p)
			else:
				if(os.path.isdir(p)):
					images_paths = os.listdir(p)
					directory = p+"/"
					# Add all images from the directory
					for path in images_paths:
						if(is_image(path)):
							final_paths.append(directory+path)
				else:
					print _("The file \"%s\" is not a valid file.") % p
		self.add_files_to_iconview(final_paths)
		

	# Enable the aply button if the combobox change
	def on_combo_change(self, widget):
		liste = self.iconview.get_selected_items()
		if len(liste) != 0:
			self.button_apply.set_sensitive(True)
		else:
			self.button_apply.set_sensitive(False)

	# On checkbutton activate
	def on_checkbutton_click(self, check):
		if check.get_active() == True:
			self.button_apply.set_sensitive(True)
		else:
			liste = self.iconview.get_selected_items()
			if len(liste) != 0:
				self.button_apply.set_sensitive(True)
			else:
				self.button_apply.set_sensitive(False)
	
	# On button Del press, remove all selected 
	def on_key_press(self, widget, event):
		# Del key = 65535
		if event.keyval == 65535:
			self.button_remove.emit("clicked")
		elif event.keyval == 65507:
			pass

	# Update the preview image, do not display when it is not a image
	def update_preview(self, widget):
		filename = self.filesel.get_preview_filename()
		have_preview = False
		try:
			pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(filename, 128, 128)
			self.preview.set_from_pixbuf(pixbuf)
			have_preview = True
		except:
			pass
		self.filesel.set_preview_widget_active(have_preview)


	## Constructor
	def __init__(self, directory, config_file, images_paths):

		self.images_paths = []
		self.directory = directory
		self.config_file = config_file
		self.must_save = False
		
		gtk.Window.__init__(self, gtk.WINDOW_TOPLEVEL)
		self.connect("delete_event", self.evnmt_delete)
		self.connect("destroy", self.destroy)
		
		self.set_title("Wallpabox")
		self.set_border_width(10)
		self.set_size_request(650, 500)
		self.set_icon_from_file("/usr/share/pixmaps/wallpabox.png")
		# Alignments
		align_panel = gtk.Alignment(0,0.5,0,0)
		align_buttom = gtk.Alignment(0.5,0,0,0)
		align_left = gtk.Alignment(0,0,0,0)


		# Boxes
		vbox = gtk.VBox(False, 0)
		vbox_panel = gtk.VBox(False, 0)
		hbox = gtk.HBox(False, 0)
		button_box = gtk.VBox(False, 0)
		hbox_bottom = gtk.HBox(False, 0)

		# Label
		label = gtk.Label()
		label.set_markup(_("<b>Wallpapers</b>"))
		vbox.pack_start(label, False, True, 0)


		# Frame
		frame = gtk.Frame()
		frame.set_shadow_type(gtk.SHADOW_ETCHED_OUT)
		vbox.pack_start(frame, True, True, 5)


		# Quit button
		button_quit = gtk.Button(stock=gtk.STOCK_QUIT)   
		button_quit.connect_object("clicked", gtk.Widget.destroy, self)
		
		vbox_panel.pack_end(button_quit, False, False, 5)	


		# File selection widget
		self.filesel = gtk.FileChooserDialog(_("Select your or yours wallpapers"))			

		self.filesel.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
		self.filesel.add_button(gtk.STOCK_OPEN, gtk.RESPONSE_OK)
		self.filesel.set_select_multiple(True)
		# Preview
		self.preview = gtk.Image()
		self.filesel.set_preview_widget(self.preview)
		self.filesel.connect("selection-changed", self.update_preview)

		# Filter
		filter_image = gtk.FileFilter()
		filter_image.set_name(_("Image files"))
		filter_image.add_mime_type("image/jpeg")
		filter_image.add_mime_type("image/png")
			
		self.filesel.add_filter(filter_image);


		# Organize boxes
		align_panel.add(button_box)
		vbox_panel.pack_start(align_panel, True, True, 5)
		hbox.pack_start(vbox, True, True, 5)
		hbox.pack_start(vbox_panel, False, False, 5)
		self.add(hbox)


		# ScrolledWindow
		scwindow = gtk.ScrolledWindow()
		scwindow.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		frame.add(scwindow)

		# IconView & ListStore
		self.iconview = gtk.IconView()
		self.iconview.set_selection_mode(gtk.SELECTION_MULTIPLE)
		self.model = gtk.ListStore(gtk.gdk.Pixbuf)
		
		# Add thumbnails to the main window
		self.add_files_to_iconview(images_paths)
				
		self.iconview.set_model(self.model)
		self.iconview.set_pixbuf_column(0)
		
		scwindow.add(self.iconview)

		# Drag and drop
		type = [("text/uri-list", 0, 2)]
		self.iconview.drag_dest_set(gtk.DEST_DEFAULT_ALL, type, gtk.gdk.ACTION_COPY | gtk.gdk.ACTION_LINK | gtk.gdk.ACTION_MOVE)


		# File selection button
		filew = gtk.Button(stock=gtk.STOCK_ADD)
		filew.connect("clicked", self.select_file)
		button_box.pack_start(filew, False, False, 5)

		# Remove button
		self.button_remove = gtk.Button(stock=gtk.STOCK_REMOVE)
		self.button_remove.set_sensitive(False)
		self.button_remove.connect("clicked", self.on_remove_click)
		button_box.pack_start(self.button_remove, False, False, 5)

		# Choice combobox
		self.combo = gtk.combo_box_new_text() # DO NOT CHANGE ORDER
		self.combo.append_text(_("Fullscreen"))
		self.combo.append_text(_("Centered"))
		self.combo.append_text(_("Tiled"))
		self.combo.append_text(_("Maximized"))
		self.combo.set_active(0)
		hbox_bottom.pack_start(self.combo, False, False)

		# gtk.CheckButton 
		self.randomcheck = gtk.CheckButton(_("Random wallpapers"))
		hbox_bottom.pack_start(self.randomcheck, False, False)

		# Apply button
		self.button_apply = gtk.Button(stock=gtk.STOCK_APPLY)
		self.button_apply.set_sensitive(False) 
		align_buttom.add(self.button_apply)
		hbox_bottom.pack_start(align_buttom, True, False, 0)
		vbox.pack_end(hbox_bottom, False, True, 5)

		self.button_apply.connect("clicked", self.on_apply_click)

		# Show widgets
		self.show_all()
		
		# Read config file
		self.read_configfile()
		
		# Connect all signals
		self.iconview.connect("drag_data_received", self.receiveCallback)
		self.iconview.connect("selection-changed", self.on_selection_changed)
		self.iconview.connect("item-activated", self.on_double_click)
		self.iconview.connect("key-press-event", self.on_key_press)
		self.combo.connect("changed", self.on_combo_change)
		self.randomcheck.connect("clicked", self.on_checkbutton_click)


def main():
	if len(sys.argv) != 1:
		if "--help" in sys.argv or "-h" in sys.argv:
			print "Usage : wallpabox [options]\n"
			print "  Generic options :"
			print "\t-h, --help                Show help about options"
			print "\t--author                  Show author information"
			print "\t-v, --version             Show version information"
		elif "--version" in sys.argv or "-v" in sys.argv:	
			print "Wallpabox : 0.2.4"
		elif "--author" in sys.argv:
			print "Wallpabox was written by"
			print "\tNicolas Doby <nicolas.doby@gmail.com>"
		else:
			print "Unknown option."
			print "Use --help to get a list of available command line options."
	else:
		# No argument
		directory = os.path.expanduser('~')+"/.wallpabox/"
		config_file = directory + "config"
		
		if not os.path.exists(directory):
			print _("Create a new config directory")
			os.mkdir(directory, 0777)
		if not os.path.exists(config_file):
			print _("Create a new config file")
			f = open(config_file, "w")
			f.close()
			
		# Get all images from the config directory
		images_paths = os.listdir(directory)
		images_paths_final = []
		for path in images_paths:
			if(is_image(path)):
				images_paths_final.append(directory+path)

		ChangeWallpaper(directory, config_file, images_paths_final)
		gtk.main()
		
	return 0

######################################################

if __name__ == "__main__":
	sys.exit(main())
